#include "riffhead.hh"

/* STREAM HEADER */

class strh: public RiffHead
{
public:
    virtual string Read(FILE *fp, unsigned bytes);
};

string strh::Read(FILE *fp, unsigned bytes)
{
    unsigned fourcc     = ReadDW(fp); // vids, auds
    unsigned handler    = ReadDW(fp);
    unsigned flags      = ReadDW(fp);
    unsigned priority   = ReadW(fp);
    unsigned language   = ReadW(fp);
    unsigned initframes = ReadDW(fp);
    unsigned scale      = ReadDW(fp);
    const long rate_pos = ftell(fp);
    unsigned rate       = ReadDW(fp);
    unsigned start      = ReadDW(fp);
    unsigned scalfram   = ReadDW(fp);
    unsigned suggbuff   = ReadDW(fp);
    unsigned quality    = ReadDW(fp);
    unsigned size       = ReadDW(fp);
    unsigned left       = ReadW(fp);
    unsigned top        = ReadW(fp);
    unsigned right      = ReadW(fp);
    unsigned bottom     = ReadW(fp);
    
    char Buf[256];
    string result;
    result += string(Buf, sprintf(Buf,
        "%c%c%c%c(0x%08X)",
        (char)(fourcc&255),
        (char)((fourcc>>8)&255),
        (char)((fourcc>>16)&255),
        (char)((fourcc>>24)&255),
        fourcc
    ));
    if(handler)
    {
        result += string(Buf, sprintf(Buf,
            ", %c%c%c%c(0x%08X)",
            (char)(handler&255),
            (char)((handler>>8)&255),
            (char)((handler>>16)&255),
            (char)((handler>>24)&255),
            handler
        ));
    }
    
    if(fourcc == 0x73647561) // auds
    {
    }
    else if(fourcc == 0x73646976) // vids
    {
        if(IsDefined("fps"))
        {
            double new_fps = GetFieldDouble("fps");
            PatchDW(fp, rate_pos, (unsigned)(new_fps * (double)scale));
        }
        result += string(Buf, sprintf(Buf,
            ", %.2f fps, quality %u, %ux%u-%ux%u",
            rate / (double)scale,
            quality,
            left,top,right,bottom
        ));
    }
    return result;
}

static class strhadder { public: strhadder() {
    RegisterFormat("strh", new strh);
    RegisterFieldDouble("fps", "frames per second");
} } strhadder;
